@@ -27,10 +27,10 @@
 #endif
 
 #include <glib.h>
-#include <bluetooth/uuid.h>
 #include <errno.h>
 #include <adapter.h>
 
+#include "lib/uuid.h"
 #include "plugin.h"
 #include "hcid.h"
 #include "log.h"
